dnl This was heavily modified from berkeley-db.m4 included with
dnl Subversion (r5118). Originally copyright (c) 2002-2003 CollabNet.

dnl   J2_LIB_PAM()
dnl
dnl   Search for a useable version of the PAM libs in a number of common
dnl   places.
dnl
dnl   If we find a useable version, set CPPFLAGS and LIBS as
dnl   appropriate, and set the shell variable `j2_lib_pam' to
dnl   `yes'.  Otherwise, set `j2_lib_pam' to `no'.
dnl
dnl   This macro also checks for the `--with-pam=PATH' flag;
dnl   if given, the macro will use the PATH specified, and the
dnl   configuration script will die if it can't find the library.
dnl
dnl   We cache the results of individual searches under particular
dnl   prefixes, not the overall result of whether we found PAM
dnl   That way, the user can re-run the configure script with
dnl   different --with-pam switch values, without interference
dnl   from the cache.


AC_DEFUN(J2_LIB_PAM,
[
  dnl  Process the `with-pam' switch. We set the variable `places' to
  dnl  either `search', meaning we should check in a list of typical places,
  dnl  or to a single place spec.
  dnl
  dnl  A `place spec' is either:
  dnl    - the string `std', indicating that we should look for headers and
  dnl      libraries in the standard places,
  dnl    - a directory prefix P, indicating we should look for headers in
  dnl      P/include and libraries in P/lib, or
  dnl    - a string of the form `HEADER:LIB', indicating that we should look
  dnl      for headers in HEADER and libraries in LIB.
  dnl 
  dnl  You'll notice that the value of the `--with-pam' switch is a
  dnl  place spec.

  AC_ARG_WITH(pam,
    AC_HELP_STRING(--with-pam=PATH, [alternate location of PAM headers and libs]),
  [
    if test "$withval" = "yes"; then
      places=search
    else
      places="$withval"
    fi
  ],
  [
      places=search
  ])

  if test "$places" = "search"; then
    places="std"
  fi
  # Now `places' is guaranteed to be a list of place specs we should
  # search, no matter what flags the user passed.

  # Save the original values of the flags we tweak.
  J2_LIB_PAM_save_libs="$LIBS"
  J2_LIB_PAM_save_cppflags="$CPPFLAGS"

  # The variable `found' is the prefix under which we've found
  # PAM, or `not' if we haven't found it anywhere yet.
  found=not
  for place in $places; do

    LIBS="$J2_LIB_PAM_save_libs"
    CPPFLAGS="$J2_LIB_PAM_save_cppflags"
    case "$place" in
      "std" )
        description="the standard places"
      ;;
      *":"* )
        header="`echo $place | sed -e 's/:.*$//'`"
        lib="`echo $place | sed -e 's/^.*://'`"
	    CPPFLAGS="$CPPFLAGS -I$header"
      LIBS="$LIBS -L$lib -R$lib"
      description="$header and $lib"
      ;;
      * )
	    LIBS="$LIBS -L$place/lib -R$place/lib"
	    CPPFLAGS="$CPPFLAGS -I$place/include"
      description="$place"
      ;;
    esac

    # We generate a separate cache variable for each prefix
    # we search under.  That way, we avoid caching information that
    # changes if the user runs `configure' with a different set of
    # switches.
    changequote(,)
    cache_id="`echo j2_cv_lib_pam_$1_$2_$3_in_${place} \
                 | sed -e 's/[^a-zA-Z0-9_]/_/g'`"
    changequote([,])
    dnl We can't use AC_CACHE_CHECK here, because that won't print out
    dnl the value of the computed cache variable properly.
    AC_MSG_CHECKING([for PAM in $description])
    AC_CACHE_VAL($cache_id,
      [
       J2_LIB_PAM_TRY()
         eval "$cache_id=$j2_have_pam"
      ])
    result="`eval echo '$'$cache_id`"
    AC_MSG_RESULT($result)

    # If we found it, no need to search any more.
    if test "`eval echo '$'$cache_id`" = "yes"; then
      found="$place"
      break
    fi

    test "$found" != "not" && break
  done

  # Restore the original values of the flags we tweak.
  LIBS="$J2_LIB_PAM_save_libs"
  CPPFLAGS="$J2_LIB_PAM_save_cppflags"

  case "$found" in
    "not" )
	  AC_MSG_ERROR([Could not find PAM $pam_version (or higher)])
    j2_lib_pam=no
    ;;
    "std" )
      J2_PAM_INCLUDES=
      J2_PAM_LIBS="-lpam"
      j2_lib_pam=yes
    ;;
    *":"* )
      header="`echo $found | sed -e 's/:.*$//'`"
      lib="`echo $found | sed -e 's/^.*://'`"
      J2_PAM_INCLUDES="-I$header"
      J2_PAM_LIBS="-L$lib -R$lib -lpam"
      j2_lib_pam=yes
    ;;
    * )
      J2_PAM_INCLUDES="-I$found/include"
      J2_PAM_LIBS="-L$found/lib -R$found/lib -lpam"
      j2_lib_pam=yes
    ;;
  esac
])


dnl   J2_LIB_PAM_TRY()
dnl
dnl   A subroutine of J2_LIB_PAM.
dnl
dnl   Check that a new-enough version of PAM is installed.
dnl
dnl   Set the shell variable `j2_have_pam' to `yes' if we found
dnl   an appropriate version installed, or `no' otherwise.


AC_DEFUN(J2_LIB_PAM_TRY,
  [
    j2_lib_pam_try_save_libs="$LIBS"

    LIBS="$LIBS -lpam"

    dnl not trying to run, there's no point
    AC_TRY_LINK([#include "security/pam_appl.h"],[pam_start(NULL, NULL, NULL, NULL);],[j2_have_pam=yes],[j2_have_pam=no])

    LIBS="$j2_lib_pam_try_save_libs"
  ]
)
